#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <SDL/SDL.h>
#include <SDL/SDL_rotozoom.h>
#include <math.h>
#include <string.h>
#include <time.h>

#include "config.h"

struct _square_ 
{
	int rows;
	int cols;
	int value;
	bool ispresent;
};

typedef struct _square_ square;

SDL_Surface *sdl_window=NULL;
SDL_Surface *sdl_square[ROWS][COLS];
SDL_Rect sdl_square_pos[ROWS][COLS];
SDL_Rect sdl_square_bmp_pos[ROWS][COLS];

int square_value[ROWS][COLS]={{0}, {0}};

square square_selected_1;
square square_selected_2;

void sdl_free(int status)
{
	int i, j;

	for(i=0 ; i < ROWS ; i++)
		for(j=0 ; j < COLS ; j++)
			SDL_FreeSurface(sdl_square[i][j]);

	SDL_Quit();	

	printf("Exiting with status %d.\n", status);

	exit(status);
}

void show_start(void)
{
	SDL_Surface *start_image;
	SDL_Rect start_image_pos;
	double start_image_zoom_width, start_image_zoom_height;
	
	start_image=SDL_LoadBMP(SDL_START_BMP);
	start_image_pos.x=0;
	start_image_pos.y=0;

	start_image_zoom_width=(double) SDL_WINDOW_WIDTH / (double)start_image->w;
	start_image_zoom_height=(double)SDL_WINDOW_HEIGHT / (double)start_image->h;

	start_image=zoomSurface(start_image, start_image_zoom_width, start_image_zoom_height, 1);

	SDL_BlitSurface(start_image, NULL, sdl_window, &start_image_pos);

	SDL_Flip(sdl_window);

	SDL_Delay(1000);

	SDL_FillRect(start_image, NULL, SDL_WINDOW_COLOR);
	SDL_BlitSurface(start_image, NULL, sdl_window, &start_image_pos);

	SDL_FreeSurface(start_image);
}

void show_end(void)
{
	SDL_Surface *end_image;
	SDL_Rect end_image_pos;
	double end_image_zoom_width, end_image_zoom_height;

	end_image=SDL_LoadBMP(SDL_END_BMP);
	end_image_pos.x=0;
	end_image_pos.y=0;

	end_image_zoom_width=(double) SDL_WINDOW_WIDTH / (double)end_image->w;
	end_image_zoom_height=(double)SDL_WINDOW_HEIGHT / (double)end_image->h;

	end_image=zoomSurface(end_image, end_image_zoom_width, end_image_zoom_height, 1);

	SDL_BlitSurface(end_image, NULL, sdl_window, &end_image_pos);

	SDL_Flip(sdl_window);

	SDL_Delay(1000);

	SDL_FreeSurface(end_image);

	sdl_free(0);
}


bool isended(void)
{
	int i, j;

	for(i=0 ; i < ROWS ; i++)
		for(j=0 ; j < COLS ; j++)
			if(square_value[i][j] != 0)
				return false;
	return true;
}

void square_image_display(int square_rows, int square_cols)
{
	int square_image_name_value=square_value[square_rows][square_cols];
	int square_image_name_length, i;
	double square_image_zoom_width, square_image_zoom_height;

	for((i = square_image_name_value) && (square_image_name_length=0) ; i > 0 ; square_image_name_length++)
		i-=pow(10, square_image_name_length);

	square_image_name_length+=4; //  >>/ . bmp<<
	square_image_name_length+=strlen(SDL_SQUARE_BMP_FOLDER);

	char square_image_name[square_image_name_length];

	sprintf(square_image_name, "%s/%d.bmp", SDL_SQUARE_BMP_FOLDER, square_image_name_value);

	printf("Displaying image %s.\n", square_image_name);

	sdl_square[square_rows][square_cols]=SDL_LoadBMP(square_image_name);

	square_image_zoom_width=(double) SDL_SQUARE_WIDTH / (double)sdl_square[square_rows][square_cols]->w;
	square_image_zoom_height=(double)SDL_SQUARE_HEIGHT / (double)sdl_square[square_rows][square_cols]->h;

	sdl_square[square_rows][square_cols]=zoomSurface(sdl_square[square_rows][square_cols], square_image_zoom_width, square_image_zoom_height, 1);

    SDL_BlitSurface(sdl_square[square_rows][square_cols], NULL, sdl_window, &sdl_square_pos[square_rows][square_cols]);

    SDL_Flip(sdl_window);
}

void square_image_reset(int square_rows, int square_cols, bool square_refresh)
{
	double square_image_zoom_width, square_image_zoom_height;

	sdl_square[square_rows][square_cols]=SDL_LoadBMP(SDL_SQUARE_BMP);

	square_image_zoom_width=(double) SDL_SQUARE_WIDTH / (double)sdl_square[square_rows][square_cols]->w;
	square_image_zoom_height=(double)SDL_SQUARE_HEIGHT / (double)sdl_square[square_rows][square_cols]->h;

	sdl_square[square_rows][square_cols]=zoomSurface(sdl_square[square_rows][square_cols], square_image_zoom_width, square_image_zoom_height, 1);

	SDL_BlitSurface(sdl_square[square_rows][square_cols], NULL, sdl_window, &sdl_square_pos[square_rows][square_cols]);

	if(square_refresh)
		SDL_Flip(sdl_window);
}

void square_value_set(void)
{
	int square_value_rand;
	int square_value_score;
	int i, j;
	int k, l;

	srand(time(NULL));

	for(i=0 ; i < ROWS ; i++)
	{
		for(j=0 ; j < COLS ; j++)
		{
			do
			{
				square_value_score=0;
				square_value_rand=(rand() % (COLS*ROWS/2)) + 1;

				l=j-1;

				for(k=i ; k >= 0 ; (k--) && (l=COLS-1))
				{
					while(l >= 0)
					{
						//printf("--- [%d][%d]\n", k, l);

						if(square_value[k][l] == square_value_rand)
							square_value_score++;

						l--;
					}
				}

				//printf("/// end (%dx%d)///\n", i, j);
			} while(square_value_score >= 2);

			square_value[i][j]=square_value_rand;
		}
	}

	for(i=0 ; i < ROWS ; i++)
	{
		for(j=0 ; j < COLS ; j++)
		{
			printf("Debug : square_value[%d][%d]=%d;\n", i, j, square_value[i][j]);
		}
	}
}

bool file_exists(char *file_name)
{
	FILE *file=NULL;
	file=fopen(file_name, "r");

	if(file)
	{
		fclose(file);
		return true;
	}
	else
		return false;
}

bool check_everything(void)
{
	int i;
	int square_image_name_length;

	if((COLS * ROWS) % 2 != 0)
		return false;

	if(!file_exists(SDL_SQUARE_BMP))
		return false;

	if(!file_exists(SDL_START_BMP))
		return false;

	if(!file_exists(SDL_END_BMP))
		return false;

	for((i = (COLS*ROWS) / 2) && (square_image_name_length=0) ; i > 0 ; square_image_name_length++)
		i-=pow(10, square_image_name_length);

	square_image_name_length+=4; /* "/(*).bmp" */
	square_image_name_length+=strlen(SDL_SQUARE_BMP_FOLDER);

	char square_image_name[square_image_name_length];

	for(i=1 ; i <= (COLS*ROWS) / 2 ; i++)
	{
		sprintf(square_image_name, "%s/%d.bmp", SDL_SQUARE_BMP_FOLDER, i);
		printf("Checking file %s : ", square_image_name);

		if(!file_exists(square_image_name))
		{
			printf("doesn't exist.\n");
			return false;
		}

		printf("exists.\n");
	}

  return true;
}

void square_select(int sdl_mouse_x, int sdl_mouse_y)
{
	int i;
	int square_col, square_row;

	if(sdl_mouse_x == 0 || sdl_mouse_y == 0)
		 return;

	for(i=0 ; sdl_mouse_x > 0 ; i++)
	{
		sdl_mouse_x-=SDL_SQUARE_WIDTH+2;
		
		if(sdl_mouse_x == -1 || sdl_mouse_x == 0 || sdl_mouse_x == 1)
			return;
	}

	square_col=i-1;

	for(i=0 ; sdl_mouse_y > 0 ; i++)
	{
		sdl_mouse_y-=SDL_SQUARE_HEIGHT+2;

		if(sdl_mouse_y == -1 || sdl_mouse_y == 0 || sdl_mouse_y == 1)
			 return;
	}

	square_row=i-1;

	if(square_value[square_row][square_col] == 0)
		 return;

	 printf("Debug : square_value[%d][%d]=%d;\n", square_row, square_col, square_value[square_row][square_col]);

	if(!square_selected_1.ispresent)
	{
		square_selected_1.rows=square_row;
		square_selected_1.cols=square_col;
		square_selected_1.value=square_value[square_row][square_col];
		square_selected_1.ispresent=true;

		square_image_display(square_row, square_col);
	}
	else if(!square_selected_2.ispresent)
	{
		square_selected_2.rows=square_row;
		square_selected_2.cols=square_col;
		square_selected_2.value=square_value[square_row][square_col];
		square_selected_2.ispresent=true;

		square_image_display(square_row, square_col);

		if(square_selected_1.value == square_selected_2.value)
		{
			square_value[square_selected_1.rows][square_selected_1.cols]=0;
			square_value[square_selected_2.rows][square_selected_2.cols]=0;

			square_selected_1.ispresent=0;
			square_selected_2.ispresent=0;

			if(isended())
			{
				SDL_Delay(1000);
				show_end();
			}
		}
		else
		{
			SDL_Delay(1000);

			square_image_reset(square_selected_1.rows, square_selected_1.cols, true);
			square_image_reset(square_selected_2.rows, square_selected_2.cols, true);

			square_selected_1.ispresent=0;
			square_selected_2.ispresent=0;
		}
	}
}

void sdl_events(void)
{
	bool sdl_quit=false;
	SDL_Event sdl_event;

	while(!sdl_quit)
    {
		SDL_WaitEvent(&sdl_event);

		switch(sdl_event.type)
        {
			case SDL_QUIT :
				sdl_quit=true;
			break;
			case SDL_KEYDOWN :
				if(sdl_event.key.keysym.sym == SDLK_ESCAPE)
					sdl_quit=true;
			break;

			case SDL_MOUSEBUTTONUP :
				square_select(sdl_event.button.x, sdl_event.button.y);
			break;
        }
    }
}

 main(int argc, char **argv)
{
	int i, j;

	SDL_Init(SDL_INIT_VIDEO);

	sdl_window=SDL_SetVideoMode(SDL_WINDOW_WIDTH, SDL_WINDOW_HEIGHT, 32, SDL_WINDOW_MODE);
	SDL_WM_SetCaption("Remember me", NULL);
	SDL_WM_SetIcon(SDL_LoadBMP(SDL_ICON_BMP), NULL);

	SDL_FillRect(sdl_window, NULL, SDL_WINDOW_COLOR);

   	if(!check_everything())
		sdl_free(1);

	square_value_set();
	show_start();

	printf("Debug : SDL_SQUARE_WIDTH=%d;\nDebug : SDL_SQUARE_HEIGHT=%d;\n", SDL_SQUARE_WIDTH, SDL_SQUARE_HEIGHT);

	for(i=0 ; i < ROWS ; i++)
	{
		for(j=0 ; j < COLS ; j++)
		{
			sdl_square_pos[i][j].y=SDL_SQUARE_HEIGHT*i + 2*i + 1;
			sdl_square_pos[i][j].x=SDL_SQUARE_WIDTH*j + 2*j + 1;

			square_image_reset(i, j, false);
		}
	}

	SDL_Flip(sdl_window);

	sdl_events();

	sdl_free(0);
}
